@@ -1,9 +1,10 @@
 package cmd
 
 // DCSO FEVER
-// Copyright (c) 2017, 2018, 2019, 2020, DCSO GmbH
+// Copyright (c) 2017, 2021, DCSO GmbH
 
 import (
+	"context"
 	"crypto/tls"
 	"io"
 	"os"
@@ -14,6 +15,7 @@ import (
 
 	"github.com/DCSO/fever/db"
 	"github.com/DCSO/fever/input"
+	"github.com/DCSO/fever/mgmt"
 	"github.com/DCSO/fever/processing"
 	"github.com/DCSO/fever/types"
 	"github.com/DCSO/fever/util"
@@ -69,6 +71,7 @@ func mainfunc(cmd *cobra.Command, args []string) {
 
 	dummyMode := viper.GetBool("dummy")
 
+	// configure metrics
 	enableMetrics := viper.GetBool("metrics.enable")
 	if err != nil {
 		log.Fatal(err)
@@ -140,12 +143,39 @@ func mainfunc(cmd *cobra.Command, args []string) {
 		}()
 	}
 
-	// Configure forwarding
-	outputSocket := viper.GetString("output.socket")
-	forward = (outputSocket != "")
-	eventTypes := viper.GetStringSlice("forward.types")
-	allTypes := viper.GetBool("forward.all")
-	util.PrepareEventFilter(eventTypes, allTypes)
+	// Get config from viper
+	var multiForwardConf processing.MultiForwardConfiguration
+	err = viper.Unmarshal(&multiForwardConf)
+	if err != nil {
+		log.Fatal(err)
+	}
+	// Keep supporting legacy config ("output" and "forward" sections)
+	if len(multiForwardConf.Outputs) == 0 {
+		log.Info("no multi-forwarder configuration, found, using legacy config")
+		outSocketPath := viper.GetString("output.socket")
+		if len(outSocketPath) > 0 {
+			multiForwardConf.Outputs = make(map[string]processing.MultiForwardOutput)
+			multiForwardConf.Outputs["default"] =
+				processing.MultiForwardOutput{
+					Socket: viper.GetString("output.socket"),
+					All:    viper.GetBool("forward.all"),
+					Types:  viper.GetStringSlice("forward.types"),
+				}
+		}
+	} else {
+		log.Info("found multi-forwarder configuration, ignoring legacy config")
+	}
+
+	if len(multiForwardConf.Outputs) > 0 {
+		forward = true
+	}
+
+	if pse != nil {
+		multiForwardConf.SubmitStats(pse)
+	}
+	multiFwdChan := make(chan types.Entry)
+	reconnectTimes := viper.GetInt("reconnect-retries")
+	multiForwardConf.Run(multiFwdChan, reconnectTimes)
 
 	// Optional profiling
 	profileFile := viper.GetString("profile")
@@ -185,14 +215,10 @@ func mainfunc(cmd *cobra.Command, args []string) {
 	s.Run(eventChan)
 
 	var forwardHandler processing.Handler
-	reconnectTimes := viper.GetInt("reconnect-retries")
 	// start forwarding
 	if forward {
-		forwardHandler = processing.MakeForwardHandler(int(reconnectTimes), outputSocket)
+		forwardHandler = processing.MakeForwardHandler(multiFwdChan)
 		fh := forwardHandler.(*processing.ForwardHandler)
-		if pse != nil {
-			fh.SubmitStats(pse)
-		}
 		rdns := viper.GetBool("active.rdns")
 		if rdns {
 			expiryPeriod := viper.GetDuration("active.rdns-cache-expiry")
@@ -202,7 +228,6 @@ func mainfunc(cmd *cobra.Command, args []string) {
 				fh.RDNSHandler.EnableOnlyPrivateIPRanges()
 			}
 		}
-		fh.Run()
 
 		stenosis := viper.GetBool("stenosis.enable")
 		if stenosis {
@@ -236,12 +261,6 @@ func mainfunc(cmd *cobra.Command, args []string) {
 
 		addFields := viper.GetStringMapString("add-fields")
 		fh.AddFields(addFields)
-
-		defer func() {
-			c := make(chan bool)
-			fh.Stop(c)
-			<-c
-		}()
 	} else {
 		// in this case we use a void handler that does nothing
 		forwardHandler = processing.MakeVoidHandler()
@@ -372,6 +391,10 @@ func mainfunc(cmd *cobra.Command, args []string) {
 		if err != nil {
 			log.Fatal(err)
 		}
+		testdomain := viper.GetString("pdns.test-domain")
+		if testdomain != "" {
+			pdc.EnableTestdataDomain(testdomain)
+		}
 		dispatcher.RegisterHandler(pdc)
 		pdc.Run()
 		defer func() {
@@ -418,6 +441,12 @@ func mainfunc(cmd *cobra.Command, args []string) {
 			}).Info("compression of flow stats")
 		}
 		ua := processing.MakeUnicornAggregator(submitter, unicornSleep, dummyMode)
+		testSrcIP := viper.GetString("flowreport.testdata-srcip")
+		testDestIP := viper.GetString("flowreport.testdata-destip")
+		testDestPort := viper.GetInt64("flowreport.testdata-destport")
+		if testSrcIP != "" && testDestIP != "" {
+			ua.EnableTestFlow(testSrcIP, testDestIP, testDestPort)
+		}
 		dispatcher.RegisterHandler(ua)
 		ua.Run()
 		defer func() {
@@ -490,8 +519,9 @@ func mainfunc(cmd *cobra.Command, args []string) {
 	// Heartbeat injector
 	enableHeartbeat := viper.GetBool("heartbeat.enable")
 	heartbeatTimes := viper.GetStringSlice("heartbeat.times")
+	heartbeatAlertTimes := viper.GetStringSlice("heartbeat.alert-times")
 	if enableHeartbeat {
-		hi, err := processing.MakeHeartbeatInjector(forwardHandler, heartbeatTimes)
+		hi, err := processing.MakeHeartbeatInjector(forwardHandler, heartbeatTimes, heartbeatAlertTimes)
 		if err != nil {
 			log.Fatal(err)
 		}
@@ -499,6 +529,25 @@ func mainfunc(cmd *cobra.Command, args []string) {
 		defer hi.Stop()
 	}
 
+	// create mgmt server
+	state := &mgmt.State{
+		BloomHandler: bloomHandler,
+	}
+	mgmtCfg := mgmt.EndpointConfigFromViper()
+	msrv, err := mgmt.NewMgmtServer(context.TODO(), mgmtCfg, state)
+	if err != nil {
+		log.Fatal(err)
+	}
+	go func() error {
+		if err := msrv.ListenAndServe(); err != nil {
+			log.WithFields(log.Fields{
+				"domain": "mgmt",
+			}).WithError(err).Error("starting gRPC server failed")
+			os.Exit(1)
+		}
+		return nil
+	}()
+
 	c := make(chan os.Signal, 1)
 	signal.Notify(c, os.Interrupt, syscall.SIGTERM, syscall.SIGINT, syscall.SIGUSR1)
 	go func() {
@@ -519,6 +568,7 @@ func mainfunc(cmd *cobra.Command, args []string) {
 				if myerr == nil {
 					os.Remove(inputSocket)
 				}
+				msrv.Stop()
 				os.Exit(1)
 			} else if sig == syscall.SIGUSR1 {
 				if bloomHandler != nil {
@@ -534,21 +584,30 @@ func mainfunc(cmd *cobra.Command, args []string) {
 	}()
 
 	// create input
-	inputChan := make(chan types.Entry)
+	inputBufferLen := viper.GetUint("input.buffer-length")
+	inputChan := make(chan types.Entry, inputBufferLen)
 	var sinput input.Input
 	inputRedis := viper.GetString("input.redis.server")
 	noUseRedisPipeline := viper.GetBool("input.redis.nopipe")
 	if len(inputRedis) > 0 {
-		sinput, err = input.MakeRedisInput(inputRedis, inputChan, int(chunkSize))
-		sinput.(*input.RedisInput).UsePipelining = !noUseRedisPipeline
-		sinput.(*input.RedisInput).SubmitStats(pse)
+		in, err := input.MakeRedisInput(inputRedis, inputChan, int(chunkSize))
+		if err != nil {
+			log.Fatal(err)
+		}
+		in.UsePipelining = !noUseRedisPipeline
+		in.SubmitStats(pse)
+		sinput = in
 	} else {
 		inputSocket := viper.GetString("input.socket")
-		sinput, err = input.MakeSocketInput(inputSocket, inputChan)
-	}
-	if err != nil {
-		log.Fatal(err)
+		bufDrop := viper.GetBool("input.buffer-drop")
+		in, err := input.MakeSocketInput(inputSocket, inputChan, bufDrop)
+		if err != nil {
+			log.Fatal(err)
+		}
+		in.SubmitStats(pse)
+		sinput = in
 	}
+
 	log.WithFields(log.Fields{
 		"input": sinput.GetName(),
 	}).Info("selected input driver")
@@ -583,6 +642,10 @@ func init() {
 	viper.BindPFlag("input.redis.server", runCmd.PersistentFlags().Lookup("in-redis"))
 	runCmd.PersistentFlags().BoolP("in-redis-nopipe", "", false, "do not use Redis pipelining")
 	viper.BindPFlag("input.redis.nopipe", runCmd.PersistentFlags().Lookup("in-redis-nopipe"))
+	runCmd.PersistentFlags().UintP("in-buffer-length", "", 500000, "input buffer length (counted in EVE objects)")
+	viper.BindPFlag("input.buffer-length", runCmd.PersistentFlags().Lookup("in-buffer-length"))
+	runCmd.PersistentFlags().BoolP("in-buffer-drop", "", true, "drop incoming events on FEVER side instead of blocking the input socket")
+	viper.BindPFlag("input.buffer-drop", runCmd.PersistentFlags().Lookup("in-buffer-drop"))
 
 	// Output options
 	runCmd.PersistentFlags().StringP("out-socket", "o", "/tmp/suri-forward.sock", "path to output socket (to forwarder), empty string disables forwarding")
